@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 (** Decode files and streams using GStreamer. *)
 
-open Stdlib
+open Extralib
 
 module GU = Gstreamer_utils
 module Img = Image.RGBA32
@@ -31,8 +31,6 @@ let log = Dtools.Log.make ["decoder";"gstreamer"]
 type gst =
   {
     bin : Gstreamer.Element.t;
-    src : Gstreamer.App_src.t;
-    src_mutex : Mutex.t; (** Mutex for feeding data. *)
     audio_sink : Gstreamer.App_sink.t option;
     video_sink : Gstreamer.App_sink.t option;
   }
@@ -40,7 +38,7 @@ type gst =
 (** Generic decoder. *)
 (* TODO: we should share some code with Ogg_decoder... *)
 module Make (Generator : Generator.S_Asio) = struct
-  let create_decoder ?(merge_tracks=false) _ ~channels mode input =
+  let create_decoder ?(merge_tracks=false) _ ~channels mode fname =
     let decode_audio = mode = `Both || mode = `Audio in
     let decode_video = mode = `Both || mode = `Video in
 
@@ -52,8 +50,8 @@ module Make (Generator : Generator.S_Asio) = struct
     let gst =
       let audio_pipeline =
         if decode_audio then
-          Printf.sprintf " t. ! queue ! %s ! %s"
-            (GU.Pipeline.decode_audio ())
+          Printf.sprintf " d. ! queue ! %s ! %s"
+            (GU.Pipeline.convert_audio ())
             (GU.Pipeline.audio_sink ~channels "audio_sink")
         else
           ""
@@ -61,19 +59,18 @@ module Make (Generator : Generator.S_Asio) = struct
       let video_pipeline =
         if decode_video then
           Printf.sprintf
-            " t. ! queue ! %s ! %s"
-            (GU.Pipeline.decode_video ())
+            " d. ! queue ! %s ! %s"
+            (GU.Pipeline.convert_video ())
             (GU.Pipeline.video_sink "video_sink")
         else
           ""
       in
       let pipeline =
-        Printf.sprintf "appsrc name=src ! tee name=t%s%s"
-          audio_pipeline video_pipeline
+        Printf.sprintf "filesrc location=%S ! decodebin name=d%s%s"
+          fname audio_pipeline video_pipeline
       in
       log#f 5 "Gstreamer pipeline: %s." pipeline;
       let bin = Gstreamer.Pipeline.parse_launch pipeline in
-      let src = Gstreamer.App_src.of_element (Gstreamer.Bin.get_by_name bin "src") in
       let audio_sink =
         if decode_audio then
           let sink = Gstreamer.App_sink.of_element (Gstreamer.Bin.get_by_name bin "audio_sink") in
@@ -90,8 +87,7 @@ module Make (Generator : Generator.S_Asio) = struct
         else
           None
       in
-      let src_mutex = Mutex.create () in
-      { bin; src; src_mutex; audio_sink; video_sink }
+      { bin; audio_sink; video_sink }
     in
 
     let width = Lazy.force Frame.video_width in
@@ -109,21 +105,6 @@ module Make (Generator : Generator.S_Asio) = struct
       ignore (Gstreamer.Element.set_state gst.bin Gstreamer.Element.State_playing)
     in
 
-    let feed_data buflen =
-      Mutex.lock gst.src_mutex;
-      let buf,buflen = input.Decoder.read buflen in
-      let buf = String.sub buf 0 buflen in
-      if buflen = 0 then
-        (
-          log#f 5 "End of stream.";
-          Gstreamer.App_src.end_of_stream gst.src
-        )
-      else
-        Gstreamer.App_src.push_buffer_string gst.src buf;
-      Mutex.unlock gst.src_mutex
-    in
-    Gstreamer.App_src.on_need_data gst.src feed_data;
-
     let decode buffer =
       if not !started then
         (
@@ -159,18 +140,15 @@ module Make (Generator : Generator.S_Asio) = struct
           let img = Img.make width  height b in
           let stream = [|img|] in
           Generator.put_video buffer [|stream|] 0 (Array.length stream)
-        )
+        );
+      GU.flush ~log gst.bin
     in
 
     let seek off =
       try
-        (* TODO: fix seek at some point... *)
-        ignore (raise Gstreamer.Failure);
-        let off =
-          Int64.of_float (Frame.seconds_of_master off *. 1000000000.)
-        in
-        let pos =
-          let pos = Gstreamer.Element.position gst.bin Gstreamer.Format.Time in
+        let off = Gstreamer_utils.time_of_master off in
+        let pos = Gstreamer.Element.position gst.bin Gstreamer.Format.Time in
+        let new_pos =
           Gstreamer.Element.seek_simple
             gst.bin
             Gstreamer.Format.Time
@@ -178,19 +156,27 @@ module Make (Generator : Generator.S_Asio) = struct
              Gstreamer.Event.Seek_flag_key_unit;
              Gstreamer.Event.Seek_flag_skip]
             (Int64.add pos off);
+          ignore(Gstreamer.Element.get_state gst.bin);
           Gstreamer.Element.position gst.bin Gstreamer.Format.Time
         in
-        Frame.master_of_seconds (Int64.to_float pos /. 1000000000.)
+        GU.flush ~log gst.bin;
+        Gstreamer_utils.master_of_time (Int64.sub new_pos pos) 
       with
-      | Gstreamer.Failure ->
-        log#f 3 "Seek failed!";
-        0
+       | exn ->
+           log#f 3 "Seek failed: %s" (Printexc.to_string exn);
+           log#f 4 "Backtrace:\n%s" (Printexc.get_backtrace ());
+           0
     in
 
+  let close () =
+    ignore(Gstreamer.Element.set_state gst.bin Gstreamer.Element.State_null);
+    GU.flush ~log gst.bin
+  in
+
     { Decoder.
       decode = decode;
       seek = seek;
-    }
+    }, close, gst.bin
 end
 
 module G = Generator.From_audio_video
@@ -217,10 +203,21 @@ let create_file_decoder filename content_type kind =
   in
   let channels = content_type.Frame.audio in
   let generator = G.create mode in
-  Buffered.file_decoder
-    filename kind
-    (D.create_decoder ~channels ~merge_tracks:true `File mode)
-    generator
+  let decoder, close, bin =
+    D.create_decoder ~channels ~merge_tracks:true `File mode filename
+  in
+  let remaining frame offset =
+    let pos =
+      Gstreamer_utils.master_of_time
+        (Gstreamer.Element.position bin Gstreamer.Format.Time)
+    in
+    let duration =
+      Gstreamer_utils.master_of_time
+        (Gstreamer.Element.duration bin Gstreamer.Format.Time)
+    in
+    duration - pos + G.length generator + Frame.position frame - offset
+  in
+  Buffered.make_file_decoder ~filename ~close ~kind ~remaining decoder generator
 
 (** Get the type of a file's content. For now it is a bit imprecise:
   * we always pretend that audio content has the expected number of
@@ -235,8 +232,10 @@ let get_type ~channels filename =
     in
     let bin = Gstreamer.Pipeline.parse_launch pipeline in
     ignore (Gstreamer.Element.set_state bin Gstreamer.Element.State_paused);
+    GU.flush ~log bin;
     let _, state, _ = Gstreamer.Element.get_state bin in
     ignore (Gstreamer.Element.set_state bin Gstreamer.Element.State_null);
+    GU.flush ~log bin;
     if state = Gstreamer.Element.State_paused then
       (
         log#f 5 "File %s has audio." filename;
@@ -262,7 +261,7 @@ let get_type ~channels filename =
       else
         0
     with
-    | Gstreamer.Failure -> 0
+    | Gstreamer.Failed -> 0
   in
   { Frame. video; audio; midi = 0 }
 
@@ -301,6 +300,7 @@ let () =
 
 (** Stream decoder *)
 
+(*
 module D_stream = Make(Generator.From_audio_video_plus)
 
 let () =
@@ -320,6 +320,7 @@ let () =
         Some (D_stream.create_decoder ~channels `Stream mode)
       else
         None)
+*)
 
 (** Metadata *)
 
@@ -335,33 +336,34 @@ let get_tags file =
     Printf.sprintf "filesrc location=\"%s\" ! decodebin ! fakesink" file
   in
   let bin = Gstreamer.Pipeline.parse_launch pipeline in
+  let bus = Gstreamer.Bus.of_element bin in
   (* Go in paused state. *)
   ignore (Gstreamer.Element.set_state bin Gstreamer.Element.State_paused);
+  GU.flush ~log bin;
   (* Wait for the state to complete. *)
   ignore (Gstreamer.Element.get_state bin);
   let ans = ref [] in
   try
     while true do
       let msg =
-        Gstreamer.Bus.pop_filtered
-          (Gstreamer.Bus.of_element bin)
-          [Gstreamer.Message.Error; Gstreamer.Message.Tag]
+        Gstreamer.Bus.pop_filtered bus [`Error;`Tag]
       in
       let msg = match msg with Some msg -> msg | None -> raise Exit in
-      let typ = Gstreamer.Message.message_type msg in
-      if typ <> Gstreamer.Message.Tag then raise Exit;
-      let tags = Gstreamer.Message.parse_tag msg in
-      List.iter
-        (fun (l,v) ->
-          match v with
-          | [v] -> ans := (l,v) :: !ans
-          | _ -> ()
-        ) tags
+      match msg.Gstreamer.Bus.payload with
+        | `Error _ -> GU.handler ~log ~on_error:(fun _ -> ()) msg; raise Exit
+        | `Tag tags ->
+            List.iter
+              (fun (l,v) ->
+                match v with
+                  | [v] -> ans := (l,v) :: !ans
+                  | _ -> ()) tags
+        | _ -> assert false
     done;
     assert false
   with
   | Exit ->
     ignore (Gstreamer.Element.set_state bin Gstreamer.Element.State_null);
+    GU.flush ~log bin;
     List.rev !ans
 
 let () = Request.mresolvers#register "GSTREAMER" get_tags
