@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ let load_image fname =
   let data = Bytes.create len in
   really_input ic data 0 len;
   close_in ic;
-  Img.of_PPM data
+  Img.of_PPM (Bytes.unsafe_to_string data)
 
 let () =
   Decoder.image_file_decoders#register "ppm"
