@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,9 @@ let encoder flac meta =
     }
   in
   let buf = Buffer.create 1024 in
-  let write = Buffer.add_string buf in
+  let write = fun chunk ->
+    Buffer.add_bytes buf chunk
+  in
   let cb = Flac.Encoder.get_callbacks write in
   let enc = Flac.Encoder.create ~comments p cb in
   let enc = ref enc in
