@@ -32,7 +32,7 @@ module type Transport_t =
     
     val file_descr_of_socket : socket -> Unix.file_descr
       
-    val read : socket -> int -> (string * int)
+    val read : socket -> int -> (bytes * int)
       
     val accept : Unix.file_descr -> (socket * Unix.sockaddr)
       
@@ -102,7 +102,7 @@ module type T =
     (* Generic *)
     val file_descr_of_socket : socket -> Unix.file_descr
       
-    val read : socket -> int -> (string * int)
+    val read : socket -> int -> (Bytes.t * int)
       
     val close : socket -> unit
       
@@ -135,7 +135,7 @@ module type T =
           method virtual relay :
             string ->
               (string * string) list ->
-                ?read: (socket -> int -> (string * int)) -> socket -> unit
+                ?read: (socket -> int -> (Bytes.t * int)) -> socket -> unit
           method virtual insert_metadata : (string, string) Hashtbl.t -> unit
           method virtual login : (string * (string -> string -> bool))
           method virtual icy_charset : string option
@@ -184,7 +184,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
         method virtual relay :
           string ->
             (string * string) list ->
-              ?read: (socket -> int -> (string * int)) -> socket -> unit
+              ?read: (socket -> int -> (Bytes.t * int)) -> socket -> unit
         method virtual insert_metadata : (string, string) Hashtbl.t -> unit
         method virtual login : (string * (string -> string -> bool))
         method virtual icy_charset : string option
@@ -203,7 +203,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
       ]
     
     let verb_of_string s =
-      match Utils.StringCompat.uppercase_ascii s with
+      match String.uppercase_ascii s with
       | "GET" -> `Get
       | "POST" -> `Post
       | "PUT" -> `Put
@@ -213,7 +213,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
       | _ -> raise Not_found
       
     let verb_or_source_of_string s =
-      match Utils.StringCompat.uppercase_ascii s with
+      match String.uppercase_ascii s with
       | "SOURCE" -> `Source
       | _ -> verb_of_string s
       
@@ -277,9 +277,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
       try
         (List.iter
            (fun (l, v) ->
-              if (Utils.StringCompat.uppercase_ascii l) = x
-              then raise (Assoc v)
-              else ())
+              if (String.uppercase_ascii l) = x then raise (Assoc v) else ())
            y;
          raise Not_found)
       with | Assoc s -> s
@@ -344,7 +342,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
         in
           Duppy.Monad.return
             (protocol, (List.nth data 1),
-             (match Utils.StringCompat.uppercase_ascii (List.nth data 2) with
+             (match String.uppercase_ascii (List.nth data 2) with
               | "HTTP/1.0" -> `Http_10
               | "HTTP/1.1" -> `Http_11
               | "ICE/1.0" -> `Ice_10
@@ -362,7 +360,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
           let sub = Pcre.exec ~rex h
           in ((Pcre.get_substring sub 1), (Pcre.get_substring sub 2)) :: l
         with | Not_found -> l in
-      let f x = Utils.StringCompat.uppercase_ascii x in
+      let f x = String.uppercase_ascii x in
       let headers = List.fold_right split_header headers [] in
       let display_headers =
         List.filter
@@ -476,8 +474,11 @@ module Make (T : Transport_t) : T with type socket = T.socket =
                        let read =
                          if chunked
                          then
-                           (let read =
-                              Http.read_chunked ~timeout: conf_timeout#get
+                           (let read connection =
+                              let (s, len) =
+                                Http.read_chunked ~timeout: conf_timeout#get
+                                  connection
+                              in ((Bytes.of_string s), len)
                             in Some (fun connection _ -> read connection))
                          else None in
                        let f () =
@@ -562,7 +563,8 @@ module Make (T : Transport_t) : T with type socket = T.socket =
       in
         Duppy.Monad.bind
           (Duppy.Monad.Io.write ?timeout: (Some conf_timeout#get)
-             ~priority: Tutils.Non_blocking h (Websocket.upgrade headers))
+             ~priority: Tutils.Non_blocking h
+             (Bytes.of_string (Websocket.upgrade headers)))
           (fun () ->
              let __pa_duppy_0 =
                Duppy.Monad.Io.exec ~priority: Tutils.Blocking h
@@ -641,7 +643,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
                                       let data = Buffer.sub binary_data 0 len
                                       in
                                         (Utils.buffer_drop binary_data len;
-                                         (data, len))) in
+                                         ((Bytes.of_string data), len))) in
                                    let f () =
                                      source#relay stype headers ~read
                                        h.Duppy.Monad.Io.socket
@@ -720,8 +722,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
                                         let in_enc =
                                           try
                                             let enc =
-                                              match Utils.StringCompat.
-                                                      uppercase_ascii
+                                              match String.uppercase_ascii
                                                       (Hashtbl.find args
                                                          "charset")
                                               with
@@ -919,7 +920,7 @@ module Make (T : Transport_t) : T with type socket = T.socket =
                             (Duppy.Monad.Io.write
                                ?timeout: (Some conf_timeout#get)
                                ~priority: Tutils.Non_blocking h
-                               "OK2\r\nicy-caps:11\r\n\r\n")
+                               (Bytes.of_string "OK2\r\nicy-caps:11\r\n\r\n"))
                             (fun () -> (* Now parsing headers *)
                                let __pa_duppy_0 =
                                  Duppy.Monad.Io.read
@@ -996,8 +997,9 @@ module Make (T : Transport_t) : T with type socket = T.socket =
                    let on_error e = (ignore (on_error e); close ())
                    in
                      Duppy.Io.write ~timeout: conf_timeout#get
-                       ~priority: Tutils.Non_blocking ~on_error ~string: s
-                       ~exec Tutils.scheduler socket
+                       ~priority: Tutils.Non_blocking ~on_error
+                       ~string: (Bytes.of_string s) ~exec Tutils.scheduler
+                       socket
                  in
                    Duppy.Monad.run ~return: reply ~raise: reply
                      (handle_client ~port ~icy h)))
@@ -1098,7 +1100,9 @@ module Make (T : Transport_t) : T with type socket = T.socket =
              ((Hashtbl.length handler.http) = 0)
          then
            (log#f 3 "Nothing more on port %i: closing sockets." port;
-            (let f in_s = (ignore (Unix.write in_s " " 0 1); Unix.close in_s)
+            (let f in_s =
+               (ignore (Unix.write in_s (Bytes.of_string " ") 0 1);
+                Unix.close in_s)
              in (List.iter f socks; Hashtbl.remove opened_ports port)))
          else ())
       
@@ -1126,7 +1130,9 @@ module Make (T : Transport_t) : T with type socket = T.socket =
              ((Hashtbl.length handler.http) = 0)
          then
            (log#f 4 "Nothing more on port %i: closing sockets." port;
-            (let f in_s = (ignore (Unix.write in_s " " 0 1); Unix.close in_s)
+            (let f in_s =
+               (ignore (Unix.write in_s (Bytes.of_string " ") 0 1);
+                Unix.close in_s)
              in (List.iter f socks; Hashtbl.remove opened_ports port)))
          else ())
       
