@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -150,7 +150,7 @@ object (self)
     let r = Unix.read fd s 0 len in
       (* TODO: recursive read ? *)
       assert (len = r) ;
-      Audio.S16LE.to_audio s 0 buf 0 (Audio.duration buf);
+      Audio.S16LE.to_audio (Bytes.unsafe_to_string s) 0 buf 0 (Audio.duration buf);
       AFrame.add_break frame (AFrame.size ())
 
 end
