@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
-open Stdlib
+open Extralib
 
 type category = Sys | Math | String | List | Bool | Pair
               | Liq | Control | Interaction | Other
@@ -633,7 +633,7 @@ let register_escape_fun ~name ~descr ~escape
   let special_chars =
     Lang.list ~t:Lang.string_t
      (List.map Lang.string
-      (List.map (Bytes.make 1)
+      (List.map (String.make 1)
         special_chars))
   in
   let escape_char p _ =
@@ -672,7 +672,7 @@ let register_escape_fun ~name ~descr ~escape
        let escape_char c =
          Lang.to_string
           (Lang.apply f ~t:Lang.string_t
-             ["",Lang.string (Bytes.make 1 c)])
+             ["",Lang.string (String.make 1 c)])
        in
        Lang.string (escape ~special_char ~escape_char s))
 
@@ -828,9 +828,9 @@ let () =
        let string = Lang.to_string (List.assoc "" p) in
        Lang.string
          (if lower then
-           Utils.StringCompat.lowercase_ascii string
+           String.lowercase_ascii string
           else
-           Utils.StringCompat.uppercase_ascii string))
+           String.uppercase_ascii string))
 
 let () =
   add_builtin "string.trim" ~cat:String
@@ -856,9 +856,9 @@ let () =
        let string = Lang.to_string (List.assoc "" p) in
        let f s =
            if cap then
-             Utils.StringCompat.capitalize_ascii s
+             String.capitalize_ascii s
            else
-             Utils.StringCompat.uncapitalize_ascii s
+             String.uncapitalize_ascii s
       in
       Lang.string
       (if space_sensitive then
@@ -1384,13 +1384,13 @@ let () =
   add_builtin "shutdown" ~cat:Sys ~descr:"Shutdown the application."
     [] Lang.unit_t
     (fun _ ->
-      Shutdown.restart := false ;
+      Configure.restart := false ;
       Tutils.shutdown () ;
       Lang.unit) ;
   add_builtin "restart" ~cat:Sys ~descr:"Restart the application."
     [] Lang.unit_t
     (fun _ ->
-      Shutdown.restart := true ;
+      Configure.restart := true ;
       Tutils.shutdown () ;
       Lang.unit);
   add_builtin "exit" ~cat:Sys
@@ -1578,7 +1578,7 @@ let () =
              let n = input ch tmp 0 1024 in
                if n = 0 then () else
                 begin
-                 Buffer.add_substring buf tmp 0 n;
+                 Buffer.add_subbytes buf tmp 0 n;
                  aux()
                 end
            in
@@ -1597,7 +1597,7 @@ let () =
              let pull buf fn =
                let bytes = Bytes.create buflen in
                let ret = fn bytes 0 buflen in
-               Buffer.add_substring buf (Bytes.to_string bytes) 0 ret;
+               Buffer.add_subbytes buf bytes 0 ret;
               `Continue
              in
              let on_stdout = pull out_buf in
@@ -1607,7 +1607,7 @@ let () =
                 status := Some s;
                 begin
                   try
-                    ignore(Unix.write in_pipe " " 0 1);
+                    ignore(Unix.write in_pipe (Bytes.of_string " ") 0 1);
                   with _ -> ()
                 end;
                 false
@@ -1741,32 +1741,6 @@ let () =
          else
            Lang.string default)
 
-let () =
-  add_builtin "server.register" ~cat:Interaction
-    ~descr:"Register a command. You can then execute this function \
-            through the server, either telnet or socket."
-    [ "namespace",Lang.string_t,Some (Lang.string ""),None ;
-      "description",Lang.string_t,
-      Some (Lang.string "No documentation available."),
-      Some "A description of your command." ;
-      "usage",Lang.string_t,Some (Lang.string ""),None ;
-      "",Lang.string_t,None,None ;
-      "",Lang.fun_t [false,"",Lang.string_t] Lang.string_t,None,None ]
-    Lang.unit_t
-    (fun p ->
-       let namespace = Lang.to_string (List.assoc "namespace" p) in
-       let descr = Lang.to_string (List.assoc "description" p) in
-       let usage = Lang.to_string (List.assoc "usage" p) in
-       let command = Lang.to_string (Lang.assoc "" 1 p) in
-       let f = Lang.assoc "" 2 p in
-       let f x =
-         Lang.to_string (Lang.apply ~t:Lang.string_t f ["",Lang.string x])
-       in
-       let ns = (Pcre.split ~pat:"\\." namespace) in
-       let usage = if usage = "" then command ^ " <variable>" else usage in
-           Server.add ~ns ~usage ~descr command f ;
-         Lang.unit)
-
 (** Data conversions. *)
 
 let () =
@@ -2207,18 +2181,18 @@ let () =
     (fun p ->
       let f = Lang.to_string (List.assoc "" p) in
       let ic = open_in f in
-      let s = ref "" in
+      let s = ref Bytes.empty in
       let buflen = 1024 in
       let buf = Bytes.create buflen in
       try
         while true do
           let n = input ic buf 0 buflen in
           if n = 0 then raise Exit;
-          s := !s ^ (if n = buflen then buf else Bytes.sub buf 0 n)
+          s := Bytes.cat !s (if n = buflen then buf else Bytes.sub buf 0 n)
         done;
         assert false
       with
-      | Exit -> close_in ic; Lang.string !s)
+      | Exit -> close_in ic; Lang.string (Bytes.unsafe_to_string !s))
 
 let () =
   add_builtin "file.watch" ~cat:Sys
