@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ let flac_gen params =
           | ("samplerate",{ term = Int i; _}) ->
               { f with Flac_format.samplerate = i }
           | ("compression",({ term = Int i; _} as t)) ->
-              if i < 0 || i >= 8 then
+              if i < 0 || i > 8 then
                 raise (Error (t,"invalid compression value")) ;
               { f with Flac_format.compression = i }
           | ("bits_per_sample",({ term = Int i; _} as t)) ->
@@ -49,9 +49,9 @@ let flac_gen params =
               { f with Flac_format.bits_per_sample = i }
           | ("bytes_per_page",{ term = Int i; _}) ->
               { f with Flac_format.fill = Some i }
-          | ("",{ term = Var s; _}) when Utils.StringCompat.lowercase_ascii s = "mono" ->
+          | ("",{ term = Var s; _}) when String.lowercase_ascii s = "mono" ->
               { f with Flac_format.channels = 1 }
-          | ("",{ term = Var s; _}) when Utils.StringCompat.lowercase_ascii s = "stereo" ->
+          | ("",{ term = Var s; _}) when String.lowercase_ascii s = "stereo" ->
               { f with Flac_format.channels = 2 }
           | (_,t) -> raise (generic_error t))
       defaults params
