@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 (* Buffered ALSA output *)
 
 open Alsa
-open Dtools
 
 class output ~kind ~clock_safe ~infallible
              ~on_stop ~on_start ~start dev source =
@@ -90,7 +89,7 @@ object (self)
                       (fun pcm buf ofs len ->
                          let sbuf = Bytes.create (2 * len * Array.length buf) in
                          Audio.S16LE.of_audio buf ofs sbuf 0 len;
-                         Pcm.writei pcm sbuf 0 len
+                         Pcm.writei pcm (Bytes.unsafe_to_string sbuf) 0 len
                       )
              );
              Pcm.set_channels dev params buffer_chans ;
