@@ -1,7 +1,7 @@
 (*****************************************************************************
   
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -194,9 +194,11 @@ module Make (T : T) =
         in
           (Bytes.set ret 0 (Char.chr pad);
            String.blit meta 0 ret 1 len;
-           if ret <> c.latest_meta
-           then (c.latest_meta <- ret; ret)
-           else "\000") in
+           let ret = Bytes.unsafe_to_string ret
+           in
+             if ret <> c.latest_meta
+             then (c.latest_meta <- ret; ret)
+             else "\000") in
       let rec process meta rem data =
         let pos = c.metaint - c.metapos in
         let before = String.sub data 0 pos in
@@ -248,7 +250,8 @@ module Make (T : T) =
                            (fun () -> Duppy_m.unlock c.condition_m))
                 | Some s ->
                     Duppy.Monad.Io.write ?timeout: (Some c.timeout)
-                      ~priority: Tutils.Non_blocking c.handler s)
+                      ~priority: Tutils.Non_blocking c.handler
+                      (Bytes.of_string s))
                (fun () ->
                   let __pa_duppy_0 =
                     Duppy.Monad.Io.exec ~priority: Tutils.Maybe_blocking
@@ -297,7 +300,7 @@ module Make (T : T) =
                           let out_enc =
                             match encoding with
                             | "" -> if icy then "ISO-8859-1" else "UTF-8"
-                            | s -> Utils.StringCompat.uppercase_ascii s in
+                            | s -> String.uppercase_ascii s in
                           let f = Configure.recode_tag ~out_enc in
                           let meta = Hashtbl.create (Hashtbl.length m)
                           in
