@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,13 @@
 
  *****************************************************************************)
 
-module Input_https = Http_source.Make(Https)
+module Config =
+struct
+  module Http = Https
+  let url_expr = Str.regexp "^[Hh][Tt][Tt][Pp][sS]://\\([^/]+\\)\\(/.*\\)?$"
+end
+
+module Input_https = Http_source.Make(Config)
 
 let () =
   Input_https.register "https"
