@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -225,7 +225,7 @@ let register_plugins () =
       List.iter add plugin_dirs ;
       List.rev !ans
   in
-    List.iter register_plugin plugins
+  List.iter (register_plugin ~log_errors:false) plugins
 
 let dssi_init =
   let inited = ref false in
