@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ object (self)
           let c =
             let c = Bytes.create 1 in
               ignore (Unix.read Unix.stdin c 0 1);
-              c.[0]
+              Bytes.get c 0
           in
             begin try
               self#log#f 3 "Playing note %d." (note_of_char c);
