@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -101,8 +101,7 @@ let dynlink_list = Hashtbl.create 2
 
 exception Done of string
 
-(* A function to load external libraries (currently
- * lame) *)
+(* A function to load external libraries (currently lame) *)
 let load_dynlinks () =
   let rec check_list f cur =
     match cur with
@@ -131,7 +130,7 @@ let load_dynlinks () =
       with
         | Dynlink.Error e ->
             dyn_log#f 3 "Error while loading dynamic %s at %s" name path;
-            dyn_log#f 4 "%s" (Dynlink.error_message e)) dynload.path;
+            dyn_log#f 3 "%s" (Dynlink.error_message e)) dynload.path;
      dyn_log#f 3 "Could not find dynamic module for %s." name
     with
       | Done path ->
@@ -139,8 +138,3 @@ let load_dynlinks () =
           dynload.load ()
   in
   Hashtbl.iter load_library dynlink_list
-
-let () =
-  Configure.at_init (fun () ->
-    load_dynlinks ();
-    load_plugins_dir Configure.plugins_dir)
