@@ -2,7 +2,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2017 Savonet team
+  Copyright 2003-2018 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -141,6 +141,62 @@ let remove ~ns cmd =
   Tutils.mutexify lock (fun () -> Hashtbl.remove commands (prefix_ns cmd ns))
     ()
   
+(* That's if you want to have your command wait. *)
+type condition =
+  { wait : (unit -> string) -> unit; signal : unit -> unit;
+    broadcast : unit -> unit
+  }
+
+module Mutex_control =
+  struct
+    type priority = Tutils.priority
+    
+    let scheduler = Tutils.scheduler
+      
+    let priority = Tutils.Non_blocking
+      
+  end
+  
+module Duppy_m = Duppy.Monad.Mutex.Factory(Mutex_control)
+  
+module Duppy_c = Duppy.Monad.Condition.Factory(Duppy_m)
+  
+type server_condition =
+  { condition : Duppy_c.condition; mutex : Duppy_m.mutex;
+    resume : unit -> string
+  }
+
+exception Server_wait of server_condition
+  
+let condition () =
+  let mutex = Duppy_m.create () in
+  let condition = Duppy_c.create () in
+  let wait resume =
+    raise
+      (Server_wait { mutex = mutex; condition = condition; resume = resume; }) in
+  let signal () =
+    Duppy.Monad.run ~return: (fun () -> ()) ~raise: (fun exn -> raise exn)
+      (Duppy_c.signal condition) in
+  let broadcast () =
+    Duppy.Monad.run ~return: (fun () -> ()) ~raise: (fun exn -> raise exn)
+      (Duppy_c.broadcast condition)
+  in { wait = wait; signal = signal; broadcast = broadcast; }
+  
+type ('a, 'b) interruption = { payload : 'a; after : 'b -> string }
+
+type write = (string, unit) interruption
+
+exception Write of write
+  
+let write ~after payload =
+  raise (Write { payload = payload; after = after; })
+  
+type read = (Duppy.Io.marker, string) interruption
+
+exception Read of read
+  
+let read ~after payload = raise (Read { payload = payload; after = after; })
+  
 let unregister ns =
   Tutils.mutexify lock
     (fun () ->
@@ -217,7 +273,9 @@ let exec s =
     try
       let (command, _, _) = Tutils.mutexify lock (Hashtbl.find commands) s
       in command args
-    with | Exit -> raise Exit
+    with | Server_wait opts -> raise (Server_wait opts)
+    | Write opts -> raise (Write opts) | Read opts -> raise (Read opts)
+    | Exit -> raise Exit
     | Not_found ->
         "ERROR: unknown command, type \"help\" to get a \
                       list of commands."
@@ -251,10 +309,29 @@ let handle_client socket ip =
     in
       Duppy.Monad.bind __pa_duppy_0
         (fun req ->
+           let rec run exec =
+             try Duppy.Monad.return (exec ())
+             with
+             | Server_wait opts ->
+                 Duppy.Monad.bind (Duppy_c.wait opts.condition opts.mutex)
+                   (fun () -> run opts.resume)
+             | Write opts ->
+                 Duppy.Monad.bind
+                   (Duppy.Monad.Io.write ?timeout: (Some (get_timeout ()))
+                      ~priority: Tutils.Non_blocking h
+                      (Bytes.of_string opts.payload))
+                   (fun () -> run opts.after)
+             | Read opts ->
+                 let __pa_duppy_0 =
+                   Duppy.Monad.Io.read ?timeout: (Some (get_timeout ()))
+                     ~priority: Tutils.Non_blocking ~marker: opts.payload h
+                 in
+                   Duppy.Monad.bind __pa_duppy_0
+                     (fun ret -> run (fun () -> opts.after ret))
+             | e -> Duppy.Monad.raise e in
            let __pa_duppy_0 =
              Duppy.Monad.Io.exec ~priority: Tutils.Maybe_blocking h
-               (try Duppy.Monad.return (exec req)
-                with | e -> Duppy.Monad.raise e)
+               (run (fun () -> exec req))
            in
              Duppy.Monad.bind __pa_duppy_0
                (fun ans ->
@@ -263,11 +340,13 @@ let handle_client socket ip =
                        (Duppy.Monad.Io.write
                           ?timeout:
                             (Some (* "BEGIN\r\n"; *) (get_timeout ()))
-                          ~priority: Tutils.Non_blocking h ans)
+                          ~priority: Tutils.Non_blocking h
+                          (Bytes.of_string ans))
                        (fun () ->
                           Duppy.Monad.Io.write
                             ?timeout: (Some (get_timeout ()))
-                            ~priority: Tutils.Non_blocking h "\r\nEND\r\n"))
+                            ~priority: Tutils.Non_blocking h
+                            (Bytes.of_string "\r\nEND\r\n")))
                     (fun () -> Duppy.Monad.return ()))) in
   let close () = try Unix.close socket with | _ -> () in
   let rec run () =
@@ -287,7 +366,7 @@ let handle_client socket ip =
           in
             Duppy.Io.write ~timeout: (get_timeout ())
               ~priority: Tutils.Non_blocking ~on_error ~exec Tutils.scheduler
-              ~string: msg socket
+              ~string: (Bytes.of_string msg) socket
       | _ ->
           (log#f 3 "Client %s disconnected without saying goodbye..!" ip;
            close ())
@@ -328,7 +407,7 @@ let start_socket () =
      Unix.bind sock bind_addr;
      Unix.listen sock max_conn;
      ignore
-       (Shutdown.at_stop
+       (Init.make ~after: [ Tutils.scheduler_shutdown_atom ]
           (fun () ->
              (log#f 3 "Unlink %s" socket_name; Unix.unlink socket_path)));
      Unix.chmod socket_path rights;
@@ -348,9 +427,10 @@ let start_telnet () =
   let () =
     (* The socket has to be closed for restart to work, and this has to be
        done after duppy has stopped using it. *)
-    Shutdown.at_stop ~name: "Server shutdown"
-      ~depends: [ Shutdown.duppy_scheduler () ]
-      (fun () -> (log#f 3 "Closing socket."; Unix.close sock))
+    ignore
+      (Init.make ~after: [ Tutils.scheduler_shutdown_atom ]
+         ~name: "Server shutdown"
+         (fun () -> (log#f 3 "Closing socket."; Unix.close sock)))
   in
     (* Set TCP_NODELAY on the socket *)
     (Unix.setsockopt sock Unix.TCP_NODELAY true;
